"""
Provides the recommendations for purchasing the savings plan
"""
import logging
import pandas as pd

logger = logging.getLogger()


class sp:
    def sp_recommendations(self) -> list:
        """
        :return: list of savings plan recommendations
        """
        logger.info(" ---Inside _savings_plan :: sp :: sp_recommendations()--- ")

        # response recommendations list
        res_recommendations = []

        # creation of aws client for cost explorer
        client = self.session.client('ce')

        savings_plan_type = ['COMPUTE_SP', 'EC2_INSTANCE_SP', 'SAGEMAKER_SP']
        terms = ['ONE_YEAR', 'THREE_YEARS']
        payment_options = ['NO_UPFRONT', 'PARTIAL_UPFRONT', 'ALL_UPFRONT']

        lookback_period = 'SIXTY_DAYS'
        uid = 1
        for sp_type in savings_plan_type:
            logger.info('Savings Plan type: ' + sp_type)
            for term in terms:
                logger.info('term: ' + term)
                for payment_option in payment_options:
                    logger.info('PaymentOption: ' + payment_option)
                    marker = ''
                    while True:
                        if marker == '':
                            response = client.get_savings_plans_purchase_recommendation(
                                SavingsPlansType=sp_type,
                                TermInYears=term,
                                PaymentOption=payment_option,
                                LookbackPeriodInDays=lookback_period
                            )
                        else:
                            response = client.get_savings_plan_purchase_recommendation(
                                SavingsPlansType=sp_type,
                                TermInYears=term,
                                PaymentOption=payment_option,
                                LookbackPeriodInDays=lookback_period,
                                NextPageToken=marker
                            )
                        if 'SavingsPlansPurchaseRecommendationDetails' in response[
                            'SavingsPlansPurchaseRecommendation']:
                            for recommendation in response['SavingsPlansPurchaseRecommendation']['SavingsPlansPurchaseRecommendationDetails']:
                                temp = {
                                    'AccountScope': response['SavingsPlansPurchaseRecommendation']['AccountScope'],
                                    'SavingsPlansType': response['SavingsPlansPurchaseRecommendation']['SavingsPlansType'],
                                    'TermInYears': response['SavingsPlansPurchaseRecommendation']['TermInYears'],
                                    'PaymentOption': response['SavingsPlansPurchaseRecommendation']['PaymentOption'],
                                    'LookbackPeriodInDays': response['SavingsPlansPurchaseRecommendation']['LookbackPeriodInDays'],
                                    'SavingsPlansDetails': recommendation['SavingsPlansDetails'],
                                    'AccountId': recommendation['AccountId'],
                                    'UpfrontCost': recommendation['UpfrontCost'],
                                    'EstimatedROI': recommendation['EstimatedROI'],
                                    'CurrencyCode': recommendation['CurrencyCode'],
                                    'EstimatedSPCost': recommendation['EstimatedSPCost'],
                                    'EstimatedOnDemandCost': recommendation['EstimatedOnDemandCost'],
                                    'EstimatedOnDemandCostWithCurrentCommitment': recommendation['EstimatedOnDemandCostWithCurrentCommitment'],
                                    'EstimatedSavingsAmount': recommendation['EstimatedSavingsAmount'],
                                    'EstimatedSavingsPercentage': recommendation['EstimatedSavingsPercentage'],
                                    'HourlyCommitmentToPurchase': recommendation['HourlyCommitmentToPurchase'],
                                    'EstimatedAverageUtilization': recommendation['EstimatedAverageUtilization'],
                                    'EstimatedMonthlySavingsAmount': recommendation['EstimatedMonthlySavingsAmount'],
                                    'CurrentMinimumHourlyOnDemandSpend': recommendation['CurrentMinimumHourlyOnDemandSpend'],
                                    'CurrentMaximumHourlyOnDemandSpend': recommendation['CurrentMaximumHourlyOnDemandSpend'],
                                    'CurrentAverageHourlyOnDemandSpend': recommendation['CurrentAverageHourlyOnDemandSpend']
                                }
                                res_recommendations.append(temp)

                        try:
                            marker = response['NextPageToken']
                            if marker == '' or marker is None:
                                break
                        except KeyError:
                            break

        return res_recommendations



import logging
from aws_cost_optimization_14.utils import *

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


class cw:
    def logs_costing(self, data) -> dict:
        """
        :return: cost details for logs costing
        """
        logger.info(" ---Inside _cloudwatch_costing :: cw :: logs_costing--- ")

        region = data['Metadata']['Region']

        resolved_region = self.aws_region_map[region]

        filters = [
            {'Type': 'TERM_MATCH', 'Field': 'productFamily', 'Value': 'Storage Snapshot'},
            {'Type': 'TERM_MATCH', 'Field': 'location', 'Value': resolved_region}
        ]
        price = get_pricing(self.session, region, 'AmazonCloudWatch', Filters=filters, service_name='cwlog')

        stored_bytes = data['Metadata']['storedBytes']

        stored_gb = int(stored_bytes)/(1024**3)

        # cost_gb = stored_gb - 5
        cost_gb = stored_gb
        current_cost = float(price['AmazonCloudWatch']) * cost_gb
        effective_cost = 0

        try:
            savings_p = ((current_cost - effective_cost) / current_cost) * 100
        except ZeroDivisionError:
            savings_p = 0

        return {
            'Current Cost': current_cost,
            'Effective Cost': effective_cost,
            'Savings': current_cost - effective_cost,
            'Savings %': savings_p,
            'Exception': ''
        }




import time
import traceback

from boto3 import session
import logging

from aws_cost_optimization_14 import _savings_plan, _ri_recommendations, _ec2_costing, _rds_costing, _kms_costing, \
    _cloudwatch_costing, _elb_costing
from aws_cost_optimization_14.utils import *

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()

__author__ = "Dheeraj Banodha"
__version__ = '0.2.13'


class aws_client(_ri_recommendations.ri, _savings_plan.sp, _ec2_costing.ec2, _rds_costing.rds, _kms_costing.kms, _cloudwatch_costing.cw, _elb_costing.elb):
    def __init__(self, **kwargs):
        if 'aws_access_key_id' in kwargs.keys() and 'aws_secret_access_key' in kwargs.keys():
            self.session = session.Session(
                aws_access_key_id=kwargs['aws_access_key_id'],
                aws_secret_access_key=kwargs['aws_secret_access_key'],
            )
        elif 'profile_name' in kwargs.keys():
            self.session = session.Session(profile_name=kwargs['profile_name'])

        self.regions = get_regions(self.session)
        self.aws_region_map = {
            'us-west-2': 'US West (Oregon)',
            'us-west-1': 'US West (N. California)',
            'us-east-2': 'US East (Ohio)',
            'us-east-1': 'US East (N. Virginia)',
            'sa-east-1': 'South America (Sao Paulo)',
            'eu-west-1': 'EU (Ireland)',
            'eu-west-3': 'EU (Paris)',
            'eu-west-2': 'EU (London)',
            'eu-central-1': 'EU (Frankfurt)',
            'eu-north-1': 'EU (Stockholm)',
            'ca-central-1': 'Canada (Central)',
            'ap-south-1': 'Asia Pacific (Mumbai)',
            'ap-southeast-2': 'Asia Pacific (Sydney)',
            'ap-southeast-1': 'Asia Pacific (Singapore)',
            'ap-northeast-3': 'Asia Pacific (Osaka)',
            'ap-northeast-2': 'Asia Pacific (Seoul)',
            'ap-northeast-1': 'Asia Pacific (Tokyo)',
            'me-south-1': 'Middle East (Bahrain)',
            'me-central-1': 'Middle East (UAE)',
            'eu-south-2': 'Europe (Spain)',
            'eu-south-1': 'Europe (Milan)',
            'eu-central-2': 'Europe (Zurich)',
            'ap-southeast-4': 'Asia Pacific (Melbourne)',
            'ap-southeast-3': 'Asia Pacific (Jakarta)',
            'ap-south-2': 'Asia Pacific (Hyderabad)',
            'ap-east-1': 'Asia Pacific (Hong Kong)',
            'af-south-1': 'Africa (Cape Town)',
            'us-gov-west-1': 'AWS GovCloud (US)',
        }

    # returns the cost details of recommendations
    def cost_details(self, data: dict) -> dict:
        """
        :param data:
        :return:
        """
        available_cost_details = {
            'Delete idle EBS volume': self.unused_ebs_costing,
            'Associate the EIP with a running active instance, or release the unassociated EIP': self.unallocated_eip,
            'Migrate GP2 volume to GP3': self.gp2_to_gp3,
            'Remove unused EBS volume': self.unused_ebs_costing,
            'Purge unattached volume': self.unused_ebs_costing,
            'Purge 8 week older snapshot': self.older_snapshot_costing,
            # 'Remove AMI': None,
            # 'Remove Unused ELB': None,
            'Remove Customer Master Key': self.cmk_cost,
            'Delete idle rds instance': self.delete_rds_costing,
            'Upgrade to General Purpose SSD': self.rds_gp_ssd,
            # 'Delete idle compute instance': self.remove_ec2_costing,
            'Add retention period in log group': self.logs_costing,
            'Upgrade Storage Type': self.upgrade_ebs_costing,
            'Terminate Elastic Load Balancer': self.elb_costing
        }
        if data['Recommendation'] in available_cost_details.keys():
            try:
                logger.info(data)
                response = available_cost_details[data['Recommendation']](data)
            except Exception as e:
                logger.info('-------------Exception-------------')
                logger.info(e)
                response = {
                    'Current Cost': 0,
                    'Effective Cost': 0,
                    'Savings': 0,
                    'Savings %': 0,
                    'Exception': str(traceback.format_exc())
                }
        else:
            logger.info('cost details not included in '+data['Recommendation'])
            response = {
                'Current Cost': 0,
                'Effective Cost': 0,
                'Savings': 0,
                'Savings %': 0,
                'Exception': 'Not included'
            }

        return response



"""
Returns the recommendation for purchasing the Reserved instance
"""
import logging

import pandas as pd

logger = logging.getLogger()


class ri:
    def ri_recommendations(self) -> list:
        """
        :return: list of recommendations for purchasing the savings plan
        """
        logger.info(" ---Inside _ri_recommendations:: ri :: ri_recommendations()--- ")

        client = self.session.client('ce')

        res_recommendations = []

        service_map = {
            'Amazon Elastic Compute Cloud - Compute': 'EC2InstanceDetails',
            'Amazon Relational Database Service': 'RDSInstanceDetails',
            'Amazon Redshift': 'RedshiftInstanceDetails',
            'Amazon ElastiCache': 'ElastiCacheInstanceDetails',
            'Amazon Elasticsearch Service': 'ESInstanceDetails',
            'Amazon OpenSearch Service': ''
        }
        for service in service_map:
            logger.info("Service: "+service)
            marker = ''
            while True:
                if marker == '':
                    response = client.get_reservation_purchase_recommendation(
                        Service=service,
                        LookbackPeriodInDays='SIXTY_DAYS'
                    )
                else:
                    response = client.get_reservation_purchase_recommendation(
                        Service=service,
                        LookbackPeriodInDays='SIXTY_DAYS',
                        NextPageToken=marker
                    )
                for recommendation in response['Recommendations']:
                    logger.info(len(recommendation['RecommendationDetails']))
                    for recommendation_details in recommendation['RecommendationDetails']:
                        instance_detail = recommendation_details['InstanceDetails'][service_map[service]] \
                            if not service == 'Amazon OpenSearch Service' else ''

                        temp = {
                            'AccountScope':recommendation['AccountScope'],
                            'LookbackPeriodInDays': recommendation['LookbackPeriodInDays'],
                            'TermInYears': recommendation['TermInYears'],
                            'PaymentOption': recommendation['PaymentOption'],
                            'ServiceSpecification': recommendation['ServiceSpecification'] if service == 'Amazon Elastic Compute Cloud - Compute' else '',
                            'AccountId': recommendation_details['AccountId'],
                            'Details': instance_detail,
                            'RecommendedNumberOfInstancesToPurchase': recommendation_details['RecommendedNumberOfInstancesToPurchase'],
                            'RecommendedNormalizedUnitsToPurchase': recommendation_details['RecommendedNormalizedUnitsToPurchase'],
                            'MinimumNumberOfInstancesUsedPerHour': recommendation_details['MinimumNumberOfInstancesUsedPerHour'],
                            'MinimumNormalizedUnitsUsedPerHour': recommendation_details['MinimumNormalizedUnitsUsedPerHour'],
                            'MaximumNumberOfInstancesUsedPerHour': recommendation_details['MaximumNumberOfInstancesUsedPerHour'],
                            'MaximumNormalizedUnitsUsedPerHour': recommendation_details['MaximumNormalizedUnitsUsedPerHour'],
                            'AverageNumberOfInstancesUsedPerHour': recommendation_details['AverageNumberOfInstancesUsedPerHour'],
                            'AverageNormalizedUnitsUsedPerHour': recommendation_details['AverageNormalizedUnitsUsedPerHour'],
                            'AverageUtilization': recommendation_details['AverageUtilization'],
                            'EstimatedBreakEvenInMonths': recommendation_details['EstimatedBreakEvenInMonths'],
                            'CurrencyCode': recommendation_details['CurrencyCode'],
                            'EstimatedMonthlySavingsAmount': recommendation_details['EstimatedMonthlySavingsAmount'],
                            'EstimatedMonthlySavingsPercentage': recommendation_details['EstimatedMonthlySavingsPercentage'],
                            'EstimatedMonthlyOnDemandCost': recommendation_details['EstimatedMonthlyOnDemandCost'],
                            'EstimatedReservationCostForLookbackPeriod': recommendation_details['EstimatedReservationCostForLookbackPeriod'],
                            'UpfrontCost': recommendation_details['UpfrontCost'],
                            'RecurringStandardMonthlyCost': recommendation_details['RecurringStandardMonthlyCost']
                        }
                        res_recommendations.append(temp)

                try:
                    marker = response['NextPageToken']
                    if marker is None or marker == '':
                        break
                except KeyError:
                    break

        return res_recommendations





import logging
from aws_cost_optimization_14.utils import *

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


class ec2:
    # returns the costing details of delete idle compute instance
    def remove_ec2_costing(self, data: dict) -> dict:
        """
        :param data:
        :return:
        """
        logger.info(" --- Inside aws_client :: remove_ec2_costing()--- ")

        region = data['Metadata']['Region']

        client = self.session.client('ec2', region)
        response = client.describe_instances(
            InstanceIds=[data['Id']]
        )
        tenancy = None
        for reservation in response['Reservations']:
            for instance in reservation['Instances']:
                if instance['InstanceId'] == data['Id']:
                    tenancy = instance['Placement']['Tenancy']

        if tenancy == 'host':
            tenancy_filter = 'Host'
        elif tenancy == 'dedicated':
            tenancy_filter = 'Dedicated'
        else:
            tenancy_filter = 'NA'

        byol = False

        resolved_region = self.aws_region_map[region]
        filters = [
            {'Type': 'TERM_MATCH', 'Field': 'termType', 'Value': 'OnDemand'},
            {'Type': 'TERM_MATCH', 'Field': 'capacitystatus',
             'Value': 'AllocatedHost' if tenancy == 'host' else 'Used'},
            {'Type': 'TERM_MATCH', 'Field': 'tenancy', 'Value': tenancy_filter},
            {
                'Type': 'TERM_MATCH',
                'Field': 'productFamily',
                'Value': 'Compute Instance'
            },
            {
                'Type': 'TERM_MATCH',
                'Field': 'location',
                'Value': resolved_region
            },
            {
                'Type': 'TERM_MATCH',
                'Field': 'instanceType',
                'Value': data['Metadata']['Instance Type']
            },
            {
                'Type': 'TERM_MATCH',
                'Field': 'licenseModel',
                'Value': 'Bring your own license' if byol else 'No License required'
            }
        ]
        if data['Metadata']['Platform'] is not None:
            filters.append({
                'Type': 'TERM_MATCH',
                'Field': 'operatingSystem',
                'Value': data['Metadata']['Platform']
            })
        price = get_pricing(self.session, data['Metadata']['Region'], 'AmazonEC2', filters, service_name='ec2_instance')

        print(price)

    def gp2_to_gp3(self, data: dict) -> dict:
        """
        :param data: recommendations data
        :return: cost saving recommendations
        """
        logger.info(" ---Inside aws_client :: gp2_to_gp3()--- ")

        region = data['Metadata']['Region']
        resolved_region = self.aws_region_map[region]

        Filters = [
            {'Type': 'TERM_MATCH', 'Field': 'volumeType', 'Value': 'General Purpose'},
            {'Type': 'TERM_MATCH', 'Field': 'location', 'Value': resolved_region}
        ]
        price = get_pricing(self.session, region, 'AmazonEC2', Filters=Filters, service_name='volume')

        client = self.session.client('ec2', region_name=region)
        response = client.describe_volumes(
            VolumeIds=[
                data['Id']
            ]
        )
        size = 1
        for volume in response['Volumes']:
            if volume['VolumeId'] == data['Id']:
                size = volume['Size']

        current_cost = float(price['gp2']) * float(size)
        effective_cost = float(price['gp3']) * float(size)
        recommendation = {
            'Current Cost': current_cost,
            'Effective Cost': effective_cost,
            'Savings': current_cost - effective_cost,
            'Savings %': ((current_cost - effective_cost) / current_cost) * 100,
            'Exception': ''
        }

        return recommendation

    def unallocated_eip(self, data) -> dict:
        """
        :return: this list the potential savings
        """
        logger.info(" ---Inside aws_client :: unallocated_eip()--- ")

        region = data['Metadata']['Region']

        resolved_region = self.aws_region_map[region]

        filters = [
            {'Type': 'TERM_MATCH', 'Field': 'productFamily', 'Value': 'IP Address'},
            # {'Type': 'TERM_MATCH', 'Field': 'location', 'Value': resolved_region},
            # {'Type': 'TERM_MATCH', 'Field': 'usagetype', 'Value': 'APS3-ElasticIP:IdleAddress'}
            {'Type': 'TERM_MATCH', 'Field': 'usagetype', 'Value': 'ElasticIP:IdleAddress'}
        ]
        price = get_pricing(self.session, region, 'AmazonEC2', filters, service_name='eip')
        current_cost = float(price['ElasticIP:IdleAddress']) * 730
        effective_cost = 0
        recommendation = {
            'Current Cost': current_cost,
            'Effective Cost': effective_cost,
            'Savings': current_cost - effective_cost,
            'Savings %': ((current_cost - effective_cost) / current_cost) * 100,
            'Exception': ''
        }

        return recommendation

    #   Provides cost details for recommendation unused_ebs
    def unused_ebs_costing(self, data: dict) -> dict:
        """
        :param data:
        :return:
        """
        logger.info(" ---Inside _ec2_costing.ec2 :: unused_ebs_costing()--- ")

        region = data['Metadata']['Region']

        if data['Metadata']['Instance Type'].startswith('gp'):
            volume_type = 'General Purpose'
        else:
            volume_type = "Provisioned IOPS"

        resolved_region = self.aws_region_map[region]
        filters = [
            {'Type': 'TERM_MATCH', 'Field': 'volumeType', 'Value': volume_type},
            {'Type': 'TERM_MATCH', 'Field': 'location', 'Value': resolved_region}
        ]

        price = get_pricing(self.session, data['Metadata']['Region'], 'AmazonEC2', filters, service_name='volume')

        client = self.session.client('ec2', region_name=region)
        response = client.describe_volumes(
            VolumeIds=[
                data['Id']
            ]
        )
        size = 1
        for volume in response['Volumes']:
            if volume['VolumeId'] == data['Id']:
                size = volume['Size']

        current_cost = float(price['gp2']) * float(size)
        effective_cost = 0
        recommendation = {
            'Current Cost': current_cost,
            'Effective Cost': effective_cost,
            'Savings': current_cost - effective_cost,
            'Savings %': ((current_cost - effective_cost) / current_cost) * 100,
            'Exception': ''
        }

        return recommendation

    # InComplete ******************************************
    # returns the recommendations for ec2_upgrades
    # def ec2_upgrades(self) -> list:
    #     """
    #     :return:
    #     """
    #     logger.info(" ---Inside aws_client :: rds_upgrades()--- ")
    #
    #     recommendations = []
    #
    #     ec2_instances = list_ec2_instances(self.session, self.regions)
    #
    #     for region, reservations in ec2_instances.items():
    #         resolved_region = self.aws_region_map[region]
    #         for reservation in reservations:
    #             for instance in reservation['Instances']:
    #                 instance_type = instance['InstanceType']
    #                 instance_family = instance_type.split('.')[0]
    #                 print(instance['InstanceId'])
    #                 print(instance_type)
    #
    #                 Filters = [
    #                     {'Type': 'TERM_MATCH', 'Field': 'instanceType', 'Value': instance_type},
    #                     {'Type': 'TERM_MATCH', 'Field': 'location', 'Value': resolved_region},
    #                     {'Type': 'TERM_MATCH', 'Field': 'operatingSystem', 'Value': 'Red Hat Enterprise Linux with HA'},
    #                     {'Type': 'TERM_MATCH', 'Field': 'marketoption', 'Value': 'OnDemand'}
    #                 ]
    #                 print(get_pricing(
    #                     self.session, region, 'AmazonEC2', Filters ,
    #                         service_name = 'ec2_instance
    #                 ))
    #
    #     return recommendations

    #  Provides the cost details for delete older snapshot recommendations
    def older_snapshot_costing(self, data: dict) -> dict:
        """
        :param data: recommendations details
        :return: cost and savings information
        """
        logger.info(" ---Inside _ec2_costing.ec2 :: older_snapshot_costing()--- ")

        region = data['Metadata']['Region']
        resolved_region = self.aws_region_map[region]

        client = self.session.client('ec2', region_name=region)
        response = client.describe_snapshots(
            SnapshotIds=[data['Id']]
        )

        try:
            if response['Snapshots'][0]['StorageTier'] == 'archive':
                usage_type = 'EBS:SnapshotArchiveStorage'
            else:
                usage_type = 'EBS:SnapshotUsage'
        except KeyError:
            usage_type = 'EBS:SnapshotUsage'

        filters = [
            {'Type': 'TERM_MATCH', 'Field': 'productFamily', 'Value': 'Storage Snapshot'},
            {'Type': 'TERM_MATCH', 'Field': 'usagetype', 'Value': usage_type},
            # {'Type': 'TERM_MATCH', 'Field': 'location', 'Value': resolved_region}
        ]
        price = get_pricing(self.session, region, 'AmazonEC2', Filters=filters, service_name='snapshot')
        print(price)
        size = response['Snapshots'][0]['VolumeSize']
        current_cost = float(price['EBS:SnapshotUsage']) * float(size)
        effective_cost = 0

        recommendation = {
            'Current Cost': current_cost,
            'Effective Cost': effective_cost,
            'Savings': current_cost - effective_cost,
            'Savings %': ((current_cost - effective_cost) / current_cost) * 100,
            'Exception': ''
        }

        return recommendation

    # returns the costing details for upgrading the volume type to GP
    def upgrade_ebs_costing(self, data: dict) -> dict:
        """
        :param data: recommendation details
        :return: cost details for ebs
        """
        logger.info(" ---Inside _ec2_costing :: ec2 :: upgrading_ebs_costing()--- ")

        region = data['Metadata']['Region']
        resolved_region = self.aws_region_map[region]

        filters = lambda v_type: [
            {'Type': 'TERM_MATCH', 'Field': 'volumeType', 'Value': v_type},
            {'Type': 'TERM_MATCH', 'Field': 'location', 'Value': resolved_region}
        ]

        price_provisioned = get_pricing(self.session, region, 'AmazonEC2', filters('Provisioned IOPS'), service_name='volume')

        # print(price_provisioned)
        current_cost = float(price_provisioned[data['Metadata']['Instance Type']]) * data['Metadata']['size']

        price_gp3 = get_pricing(self.session, region, 'AmazonEC2', filters('General Purpose'), service_name='volume')

        # print(price_gp3)
        effective_cost = float(price_gp3['gp3']) * data['Metadata']['size']
        savings = current_cost - effective_cost
        try:
            savings_p = ((current_cost - effective_cost) / current_cost) * 100
        except ZeroDivisionError:
            savings_p = 0

        recommendation = {
            'Current Cost': current_cost,
            'Effective Cost': effective_cost,
            'Savings': savings,
            'Savings %': savings_p,
            'Exception': ''
        }
        return recommendation





"""
Contains the methods which provides the costing details of AWS KMS service
"""

from aws_cost_optimization_14.utils import *

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


class kms:
    def cmk_cost(self, data: dict) -> dict:
        """
        :param data: recommendations data
        :return: cost details for kms cmk
        """
        logger.info(" ---Inside _kms_costing.kms :: cmk_cost")

        region = data['Metadata']['Region']
        resolved_region = self.aws_region_map[region]

        Filters = [
            {'Type': 'TERM_MATCH', 'Field': 'productFamily', 'Value': 'Encryption Key'},
            {'Type': 'TERM_MATCH', 'Field': 'location', 'Value': resolved_region}
        ]
        price = get_pricing(self.session, region, 'awskms', Filters=Filters, service_name='cmk')

        current_cost = float(price['AWS Key Management Service'])
        effective_cost = 0

        return {
            'Current Cost': current_cost,
            'Effective Cost': effective_cost,
            'Savings': current_cost - effective_cost,
            'Savings %': ((current_cost - effective_cost) / current_cost) * 100,
            'Exception': ''
        }







import json
import logging

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


def get_regions(session):
    """
    :session: aws session object
    :return: list of regions
    """
    logger.info(" ---Inside utils :: get_regions()--- ")
    client = session.client('ec2', region_name='us-east-1')
    region_response = client.describe_regions()

    regions = [region['RegionName'] for region in region_response['Regions']]
    return regions


def list_volumes(session, regions: list) -> dict:
    """
    :param regions:
    :param session:
    :return:
    """
    logger.info(" ---Inside utils :: list_gp2_volumes()--- ")

    volume_list = {}

    for region in regions:
        client = session.client('ec2', region_name=region)
        marker = ''
        while True:
            if marker == '':
                response = client.describe_volumes()
            else:
                response = client.describe_volumes(
                    NextToken=marker
                )
            volume_list.setdefault(region, []).extend(response['Volumes'])

            try:
                marker = response['NextToken']
                if marker == '':
                    break
            except KeyError:
                break

    return volume_list


# returns the pricing of resource
def get_pricing(session, region: str, service_code: str, Filters: list, service_name: str) -> dict:
    """
    :param service_name:
    :param Filters:
    :param service_code:
    :param region: aws region
    :param session: aws session
    :return: pricing
    """
    logger.info(" ---Inside utils :: get_pricing()--- ")

    aws_pricing_region = region

    client = session.client('pricing', 'us-east-1')

    response = client.get_products(
        ServiceCode=service_code,
        Filters=Filters
    )
    print(response)
    prices = {}
    for price in response['PriceList']:
        price = json.loads(price)
        # print(json.dumps(price, indent=4))
        for key in price['terms']['OnDemand'].keys():
            for k in price['terms']['OnDemand'][key]['priceDimensions'].keys():
                temp = price['terms']['OnDemand'][key]['priceDimensions'][k]['pricePerUnit']['USD']

                if service_name == 'volume':
                    prices[price['product']['attributes']['volumeApiName']] = temp
                elif service_name == 'rds' or service_name == 'ec2_instance':
                    prices[price['product']['attributes']['instanceType']] = temp
                elif service_name == 'eip':
                    if price['terms']['OnDemand'][key]['priceDimensions'][k]['endRange'] == 'Inf':
                        prices[price['product']['attributes']['usagetype']] = temp
                elif service_name == 'snapshot':
                    prices[price['product']['attributes']['usagetype']] = temp
                elif service_name == 'rds_storage':
                    prices[price['product']['attributes']['volumeName']] = temp
                elif service_name == 'cmk':
                    prices[price['product']['attributes']['servicename']] = temp
                elif service_name == 'cwlog':
                    prices[price['product']['attributes']['servicename']] = temp
                elif service_name == 'elb':
                    prices[price['product']['attributes']['groupDescription']] = temp

    return prices


# returns the list of rds instances
def list_rds_instances(session, regions: list) -> dict:
    """
    :param regions:
    :param session:
    :return:
    """
    logger.info(" ---Inside utils :: list_rds_instances()--- ")
    rds_instance_lst = {}

    for region in regions:
        client = session.client('rds', region_name=region)

        marker = ''
        while True:
            response = client.describe_db_instances(
                MaxRecords=100,
                Marker=marker
            )
            rds_instance_lst.setdefault(region, []).extend(response['DBInstances'])

            try:
                marker = response['Marker']
                if marker == '':
                    break
            except KeyError:
                break
    return rds_instance_lst


# returns the list of ec2 instances
def list_ec2_instances(session, regions: list) -> dict:
    """
    :param session:
    :param regions:
    :return:
    """
    logger.info(" ---Inside utils :: list_ec2_instances()--- ")

    instances = {}
    print('Instances')
    for region in regions:
        client = session.client('ec2', region_name=region)
        marker = ''
        while True:
            if marker == '':
                response = client.describe_instances()
            else:
                response = client.describe_instances(
                    NextToken=marker
                )
            instances.setdefault(region, []).extend(response['Reservations'])
            print(response)

            try:
                marker = response['NextToken']
                if marker == '':
                    break
            except KeyError:
                break

    return instances


# returns the list eip
def list_eip(session, regions: list) -> dict:
    """
    :param session:
    :param regions:
    :return:
    """
    logger.info(" ---Inside utils :: list_eip()--- ")

    eip_list = {}

    for region in regions:
        client = session.client('ec2', region_name=region)
        response = client.describe_addresses()
        eip_list.setdefault(region, []).extend(response['Addresses'])

    return eip_list



import logging
from aws_cost_optimization_14.utils import *

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


class elb:
    def elb_costing(self, data: dict) -> dict:
        """
        :param data:
        :return:
        """
        logger.info(" ---Inside _elb_costing :: elb :: elb_costing()--- ")

        region = data['Metadata']['Region']

        resolved_region = self.aws_region_map[region]

        # client = self.session.client('elbv2', region_name=region)
        # response = client.describe_load_balancers(
        #     Names=[data['Id']]
        # )

        filters = lambda elb_type: [
            {
                'Type': 'TERM_MATCH',
                'Field': 'productFamily',
                'Value': elb_type
            },
            {
                'Type': 'TERM_MATCH',
                'Field': 'location',
                'Value': resolved_region
            },
            {
                'Type': 'TERM_MATCH',
                'Field': 'locationType',
                'Value': 'AWS Region'
            }
        ]
        elbtype = data['Metadata']['Type']
        if elbtype == 'application':
            filter_elb_type = filters('Load Balancer-Application')
        elif elbtype == filters('network'):
            filter_elb_type = 'Load Balancer-Network'
        elif elbtype == 'gateway':
            filter_elb_type = filters('Load Balancer-Gateway')
        else:
            return {
                'Current Cost': 0,
                'Effective Cost': 0,
                'Savings': 0,
                'Savings %': 0
            }

        price = get_pricing(self.session, region, 'AWSELB', filter_elb_type, service_name='elb')
        current_cost = (float(price['Used Application Load Balancer capacity units-hr']) +
                        float(price['LoadBalancer hourly usage by Application Load Balancer'])) * 24 * 30
        print(price)
        effective_cost = 0
        try:
            savings_p = ((current_cost - effective_cost) / current_cost) * 100
        except ZeroDivisionError:
            savings_p = 0

        return {
            'Current Cost': current_cost,
            'Effective Cost': effective_cost,
            'Savings': current_cost - effective_cost,
            'Savings %': savings_p,
            'Exception': ''
        }



import logging
from aws_cost_optimization_14.utils import *

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


class rds:
    def rds_upgrades(self) -> list:
        """
        :return: list of cost saving recommendations
        """
        logger.info(" ---Inside aws_client :: rds_upgrades()--- ")

        recommendations = []

        rds_instances = list_rds_instances(self.session, self.regions)

        for region, rds_list in rds_instances.items():
            resolved_region = self.aws_region_map[region]
            for instance in rds_list:
                instance_type = instance['DBInstanceClass']
                instance_family = instance_type.split('.')[1]

                Filters = [
                    {'Type': 'TERM_MATCH', 'Field': 'instanceType', 'Value': instance_type},
                    {'Type': 'TERM_MATCH', 'Field': 'databaseEngine', 'Value': instance['Engine']},
                    {'Type': 'TERM_MATCH', 'Field': 'deploymentOption',
                     'Value': 'Single-AZ' if instance['MultiAZ'] else 'Multi-AZ'},
                    {'Type': 'TERM_MATCH', 'Field': 'productFamily', 'Value': 'Database Instance'},
                    {'Type': 'TERM_MATCH', 'Field': 'location', 'Value': resolved_region}
                ]

                def evaluate(frm: str, to: str):
                    price_from = get_pricing(
                        self.session, region, 'AmazonRDS',
                        Filters,
                        service_name='instanceType'
                    )
                    print(price_from)
                    Filters[0]['Value'] = instance_type.replace(frm, to)
                    price_to = get_pricing(
                        self.session, region, 'AmazonRDS', Filters,
                        service_name='rds'
                    )
                    print(price_to)
                    current_cost = float(price_from[instance_type]) * 730
                    effective_cost = float(price_to[instance_type.replace(frm, to)]) * 730

                    recommendation = {
                        'Region': region,
                        'Instance Id': instance['DBInstanceIdentifier'],
                        'Instance Type': instance_type,
                        'Upgrade To': instance_type.replace(frm, to),
                        'Current Cost': current_cost,
                        'Effective Cost': effective_cost,
                        'Savings': current_cost - effective_cost,
                        'Savings %': ((current_cost - effective_cost) / current_cost) * 100
                    }
                    return recommendation

                match instance_family:
                    case 'm3':
                        recommendations.append(evaluate('m3', 'm5'))
                    case 'r3':
                        recommendations.append(evaluate('r3', 'r5'))
                    case 'm1':
                        recommendations.append(evaluate('m1', 't2'))

        return recommendations

    def delete_rds_costing(self, data: dict) -> dict:
        """
        :param data:
        :return:
        """
        logger.info(" ---Inside aws_client :: rds_costing()--- ")

        region = data['Metadata']['Region']

        resolved_region = self.aws_region_map[region]
        filters = [
            {
                'Type': 'TERM_MATCH',
                'Field': 'productFamily',
                'Value': 'Database Instance'
            },
            {
                'Type': 'TERM_MATCH',
                'Field': 'location',
                'Value': resolved_region
            },
            {
                'Type': 'TERM_MATCH',
                'Field': 'instanceType',
                'Value': data['Metadata']['DBInstanceClass']
            },
            {
                'Type': 'TERM_MATCH',
                'Field': 'databaseEngine',
                'Value': data['Metadata']['Engine']
            },
            {
                'Type': 'TERM_MATCH',
                'Field': 'deploymentOption',
                'Value': 'Multi-AZ' if data['Metadata']['MultiAZ'] else 'Single-AZ'
            }
        ]
        price = get_pricing(self.session, data['Metadata']['Region'], 'AmazonRDS', filters, service_name='rds')
        # print(price)

        current_cost = float(price[data['Metadata']['DBInstanceClass']]) * 730
        effective_cost = 0
        recommendation = {
            'Current Cost': current_cost,
            'Effective Cost': effective_cost,
            'Savings': current_cost - effective_cost,
            'Savings %': ((current_cost - effective_cost) / current_cost) * 100,
            'Exception': ''
        }
        return recommendation

    # returns the costing details of  rds general purpose ssd
    def rds_gp_ssd(self, data: dict) -> dict:
        """
        :param data:
        :return:
        """
        logger.info(" ---Inside aws_client :: rds_gp_ssd()--- ")

        region = data['Metadata']['Region']

        resolved_region = self.aws_region_map[region]

        filters = lambda family, vtype: [
            {
                'Type': 'TERM_MATCH',
                'Field': 'productFamily',
                'Value': family
            },
            {
                'Type': 'TERM_MATCH',
                'Field': 'location',
                'Value': resolved_region
            },
            {
                'Type': 'TERM_MATCH',
                'Field': 'volumeType',
                'Value': vtype
            },
            {
                'Type': 'TERM_MATCH',
                'Field': 'databaseEdition',
                'Value': 'Standard'
            }
        ]

        price_instance = get_pricing(self.session, region, 'AmazonRDS', filters('Database Storage', 'Provisioned IOPS (SSD)'), service_name='rds_storage')
        price_iops = get_pricing(self.session, region, 'AmazonRDS', filters('Provisioned IOPS', 'Provisioned IOPS (SSD)'), service_name='rds_storage')
        current_cost = float(price_instance['io1']) + float(price_iops['io1'])
        # print(current_cost)

        gp2_price = get_pricing(self.session, region, 'AmazonRDS', filters('Database Storage', 'General Purpose (SSD)'), service_name='rds_storage')

        # print(gp2_price)
        effective_cost = float(gp2_price['gp2'])

        recommendation = {
            'Current Cost': current_cost,
            'Effective Cost': effective_cost,
            'Savings': current_cost - effective_cost,
            'Savings %': ((current_cost - effective_cost) / current_cost) * 100,
            'Exception': ''
        }
        return recommendation

